"""Filtersets for FreeIPA models."""

import django_filters
from netbox.filtersets import NetBoxModelFilterSet

from .models import FreeIPAHost


class FreeIPAHostFilterSet(NetBoxModelFilterSet):
    """Filterset for FreeIPA hosts."""
    
    fqdn = django_filters.CharFilter(
        field_name='fqdn',
        lookup_expr='icontains',
        label='FQDN'
    )
    
    enrolled = django_filters.BooleanFilter(
        field_name='enrolled',
        label='Enrolled'
    )
    
    has_keytab = django_filters.BooleanFilter(
        field_name='has_keytab',
        label='Has Keytab'
    )
    
    ip_address = django_filters.CharFilter(
        field_name='ip_address',
        lookup_expr='icontains',
        label='IP Address'
    )
    
    mac_address = django_filters.CharFilter(
        field_name='mac_address',
        lookup_expr='icontains',
        label='MAC Address'
    )
    
    class Meta:
        model = FreeIPAHost
        fields = [
            'id', 'fqdn', 'description', 'enrolled', 'has_keytab',
            'ip_address', 'mac_address'
        ]



"""Django models for NetBox FreeIPA plugin."""

from django.db import models
from django.urls import reverse
from netbox.models import NetBoxModel


class FreeIPAHost(NetBoxModel):
    """
    Model representing a host enrolled in FreeIPA.
    
    This model stores information about hosts managed by FreeIPA identity
    management system, including their enrollment status, certificates, and
    other relevant metadata.
    
    NOTE: This is a read-only mirror of FreeIPA data. No changes are made to FreeIPA.
    """
    
    clone_fields = []  # No cloning support needed for read-only data
    
    # Primary host information
    fqdn = models.CharField(
        max_length=255,
        unique=True,
        verbose_name='FQDN',
        help_text='Fully qualified domain name of the host'
    )
    
    description = models.CharField(
        max_length=200,
        blank=True,
        help_text='Description of the host'
    )
    
    # Enrollment information
    enrolled = models.BooleanField(
        default=False,
        help_text='Whether the host is currently enrolled in FreeIPA'
    )
    
    enrollment_date = models.DateTimeField(
        null=True,
        blank=True,
        help_text='Date and time when the host was enrolled'
    )
    
    # Host details from FreeIPA
    ipa_uuid = models.CharField(
        max_length=255,
        blank=True,
        verbose_name='IPA UUID',
        help_text='Unique identifier in FreeIPA'
    )
    
    mac_address = models.CharField(
        max_length=17,
        blank=True,
        verbose_name='MAC Address',
        help_text='Primary MAC address of the host'
    )
    
    ip_address = models.GenericIPAddressField(
        null=True,
        blank=True,
        verbose_name='IP Address',
        help_text='Primary IP address of the host'
    )
    
    ssh_public_key = models.TextField(
        blank=True,
        verbose_name='SSH Public Key',
        help_text='SSH public key for the host'
    )
    
    # Certificate information
    has_keytab = models.BooleanField(
        default=False,
        verbose_name='Has Keytab',
        help_text='Whether the host has a Kerberos keytab'
    )
    
    certificate_subject = models.CharField(
        max_length=500,
        blank=True,
        help_text='Certificate subject DN'
    )
    
    certificate_issuer = models.CharField(
        max_length=500,
        blank=True,
        help_text='Certificate issuer DN'
    )
    
    certificate_valid_not_before = models.DateTimeField(
        null=True,
        blank=True,
        help_text='Certificate validity start date'
    )
    
    certificate_valid_not_after = models.DateTimeField(
        null=True,
        blank=True,
        help_text='Certificate validity end date'
    )
    
    # Metadata
    managed_by = models.CharField(
        max_length=255,
        blank=True,
        help_text='Entity managing this host in FreeIPA'
    )
    
    last_synced = models.DateTimeField(
        auto_now=True,
        help_text='Last time this record was synchronized with FreeIPA'
    )
    
    class Meta:
        ordering = ['fqdn']
        verbose_name = 'FreeIPA Host'
        verbose_name_plural = 'FreeIPA Hosts'
    
    def __str__(self):
        return self.fqdn
    
    def get_absolute_url(self):
        """Return the URL to view this host."""
        return reverse('plugins:netbox_freeipa:freeipahost', args=[self.pk])
    
    @property
    def is_certificate_valid(self):
        """Check if the host's certificate is currently valid."""
        from django.utils import timezone
        now = timezone.now()
        
        if not self.certificate_valid_not_before or not self.certificate_valid_not_after:
            return None
        
        return (
            self.certificate_valid_not_before <= now <= 
            self.certificate_valid_not_after
        )
    
    @property
    def status_label(self):
        """Return a human-readable status label."""
        if not self.enrolled:
            return 'Not Enrolled'
        elif self.is_certificate_valid is False:
            return 'Certificate Expired'
        elif self.is_certificate_valid is True:
            return 'Active'
        else:
            return 'Enrolled'



"""Admin configuration for NetBox FreeIPA plugin models."""

from django.contrib import admin

from .models import FreeIPAHost


@admin.register(FreeIPAHost)
class FreeIPAHostAdmin(admin.ModelAdmin):
    """Admin interface for FreeIPAHost model."""
    
    list_display = [
        'fqdn', 'enrolled', 'ip_address', 'mac_address',
        'has_keytab', 'enrollment_date', 'last_synced'
    ]
    
    list_filter = [
        'enrolled', 'has_keytab', 'enrollment_date'
    ]
    
    search_fields = [
        'fqdn', 'description', 'ip_address', 'mac_address', 'ipa_uuid'
    ]
    
    readonly_fields = [
        'last_synced', 'created', 'last_updated'
    ]
    
    fieldsets = (
        ('Host Information', {
            'fields': (
                'fqdn', 'description', 'ipa_uuid', 'managed_by'
            )
        }),
        ('Enrollment', {
            'fields': (
                'enrolled', 'enrollment_date', 'has_keytab'
            )
        }),
        ('Network', {
            'fields': (
                'ip_address', 'mac_address'
            )
        }),
        ('Authentication', {
            'fields': (
                'ssh_public_key',
            )
        }),
        ('Certificate', {
            'fields': (
                'certificate_subject', 'certificate_issuer',
                'certificate_valid_not_before', 'certificate_valid_not_after'
            )
        }),
        ('Metadata', {
            'fields': (
                'last_synced', 'tags'
            )
        }),
    )



"""Tables for displaying FreeIPA data in NetBox."""

import django_tables2 as tables
from netbox.tables import NetBoxTable, columns

from .models import FreeIPAHost


class FreeIPAHostTable(NetBoxTable):
    """Table for displaying FreeIPA hosts."""
    
    fqdn = tables.Column(
        linkify=True,
        verbose_name='FQDN'
    )
    
    enrolled = columns.BooleanColumn(
        verbose_name='Enrolled'
    )
    
    ip_address = tables.Column(
        verbose_name='IP Address'
    )
    
    mac_address = tables.Column(
        verbose_name='MAC Address'
    )
    
    has_keytab = columns.BooleanColumn(
        verbose_name='Has Keytab'
    )
    
    enrollment_date = columns.DateTimeColumn(
        verbose_name='Enrolled On'
    )
    
    last_synced = columns.DateTimeColumn(
        verbose_name='Last Synced'
    )
    
    actions = columns.ActionsColumn(
        actions=('edit', 'delete')
    )
    
    class Meta(NetBoxTable.Meta):
        model = FreeIPAHost
        fields = (
            'pk', 'fqdn', 'description', 'enrolled', 'ip_address',
            'mac_address', 'has_keytab', 'enrollment_date', 'last_synced',
            'actions'
        )
        default_columns = (
            'pk', 'fqdn', 'description', 'enrolled', 'ip_address',
            'has_keytab', 'enrollment_date'
        )



"""Django forms for NetBox FreeIPA plugin."""

from django import forms
from netbox.forms import NetBoxModelForm, NetBoxModelFilterSetForm
from utilities.forms.fields import DynamicModelChoiceField

from .models import FreeIPAHost


class FreeIPAHostForm(NetBoxModelForm):
    """Form for creating/editing FreeIPA hosts."""
    
    class Meta:
        model = FreeIPAHost
        fields = [
            'fqdn', 'description', 'enrolled', 'enrollment_date',
            'ipa_uuid', 'mac_address', 'ip_address', 'ssh_public_key',
            'has_keytab', 'certificate_subject', 'certificate_issuer',
            'certificate_valid_not_before', 'certificate_valid_not_after',
            'managed_by', 'tags'
        ]


class FreeIPAHostFilterForm(NetBoxModelFilterSetForm):
    """Filter form for FreeIPA hosts list."""
    
    model = FreeIPAHost
    
    fqdn = forms.CharField(
        required=False,
        label='FQDN'
    )
    
    enrolled = forms.NullBooleanField(
        required=False,
        label='Enrolled',
        widget=forms.Select(choices=(
            ('', '---------'),
            ('true', 'Yes'),
            ('false', 'No'),
        ))
    )
    
    has_keytab = forms.NullBooleanField(
        required=False,
        label='Has Keytab',
        widget=forms.Select(choices=(
            ('', '---------'),
            ('true', 'Yes'),
            ('false', 'No'),
        ))
    )



"""URL configuration for NetBox FreeIPA plugin."""

from django.urls import path

from . import views


urlpatterns = [
    # Host list and detail views
    path('hosts/', views.FreeIPAHostListView.as_view(), name='freeipahost_list'),
    path('hosts/<int:pk>/', views.FreeIPAHostView.as_view(), name='freeipahost'),
    path('hosts/<int:pk>/edit/', views.FreeIPAHostEditView.as_view(), name='freeipahost_edit'),
    path('hosts/<int:pk>/delete/', views.FreeIPAHostDeleteView.as_view(), name='freeipahost_delete'),
    path('hosts/delete/', views.FreeIPAHostBulkDeleteView.as_view(), name='freeipahost_bulk_delete'),
    
    # Synchronization view
    path('sync/', views.FreeIPASyncView.as_view(), name='freeipahost_sync'),
]
